--[[
    midi2item by ePi

    使い方
        MIDIアイテムを選択した状態で実行すると,子トラックにノーツの長さ・ピッチに対応した空のアイテムを生成します.
        和音は自動でトラックを分割します.
    
    ライセンス(License)
        The MIT License
        Copyright (C) 2019 ePi

        Permission is hereby granted, free of charge, to any person obtaining a copy ofthis software and associated documentation files (the "Software"),
        to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense,
        and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

        The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

        THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
        FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,DAMAGES OR OTHER LIABILITY,
        WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
]]

local R=reaper
local function print(str,...)
    R.ShowConsoleMsg(string.format(tostring(str).."\n",...))
end

local function createNewItem(track,st,ed,pitch)
    local mi=R.AddMediaItemToTrack(track)
    local take=R.AddTakeToMediaItem(mi)
    R.SetMediaItemPosition(mi,st,false)
    R.SetMediaItemLength(mi,ed-st,false)
    R.SetMediaItemTakeInfo_Value(take,"D_PITCH",pitch) 
    return mi
end

local proj=0
R.Undo_BeginBlock2(proj)

local a,b=pcall(function()

local count=R.CountSelectedMediaItems(proj)
local before_idx=-1
local tMidi={}
for i=0,count-1 do
    local take=R.GetActiveTake(R.GetSelectedMediaItem(proj,i))
    if(R.TakeIsMIDI(take))then
        tMidi[#tMidi+1]=take
    end
end
    
local function notes_sort_comp(a,b)
    if(a[1]==b[1])then
        if(a[2]==b[2])then
            return a[3]>b[3]
        else
            return a[2]<b[2]
        end
    else
       return a[1]<b[1]
    end
end

if(#tMidi==0)then
    error("\nMIDIアイテムが選択されていません.")
end

for i=#tMidi,1,-1 do
    local midi=tMidi[i]
    local item=R.GetMediaItemTakeInfo_Value(midi,"P_ITEM")
    local midi_st=R.GetMediaItemInfo_Value(item,"D_POSITION")
    local midi_ed=midi_st+R.GetMediaItemInfo_Value(item,"D_LENGTH")
    local parent=R.GetMediaItemTake_Track(midi)
    local idx=R.GetMediaTrackInfo_Value(parent,"IP_TRACKNUMBER")

    local notes={}
    local nc=0
    for nc=0,select(2,R.MIDI_CountEvts(midi))-1 do
        local ret,_,_,st,ed,_,pitch,_=R.MIDI_GetNote(midi,nc)
        if(ret==false)then break end
        st=R.MIDI_GetProjTimeFromPPQPos(midi,st)
        ed=R.MIDI_GetProjTimeFromPPQPos(midi,ed)

        if(st<midi_st)then
            st=midi_st
            if(ed<=st)then
                goto CONTINUE
            end
        elseif(midi_ed<st)then
            goto CONTINUE
        end
        if(midi_ed<ed)then
            ed=midi_ed
        end
        notes[#notes+1]={st,ed,pitch-72}
        ::CONTINUE::
    end

    table.sort(notes,notes_sort_comp)

    local tl={{}}
    for j,note in ipairs(notes) do
        local k=1
        repeat
            local tr_ed=tl[k][#tl[k]]
            if(tr_ed==nil)then
                tl[k][1]=note
                break
            elseif(note[1]<tr_ed[2] and tr_ed[1]<note[2])then
                k=k+1
                if(tl[k]==nil)then
                    tl[k]={}
                end
            else
                tl[k][#tl[k]+1]=note
                break
            end
        until #notes<j
    end

    for j,tr in ipairs(tl) do
        local _idx=idx+j-1
        R.InsertTrackAtIndex(_idx,false)
        local track=R.GetTrack(proj,_idx)
        for k,note in ipairs(tr) do
            createNewItem(track,table.unpack(note))
        end
    end

    local depth=R.GetMediaTrackInfo_Value(parent,"I_FOLDERDEPTH")
    R.SetMediaTrackInfo_Value(parent,"I_FOLDERDEPTH",1)
    R.SetMediaTrackInfo_Value(R.GetTrack(proj,idx+#tl-1),"I_FOLDERDEPTH",depth-1)

    ::CONTINUE::
end

end)
if not(a)then
    print("エラーが発生しました:\n%s",b)
end
R.UpdateArrange()

reaper.Undo_EndBlock2(proj,"midi2item work",-1)